@@ -85,6 +85,7 @@ iTlsCertificate *   newPemKey_TlsCertificate    (const iString *certPem, const i
 
 iTlsCertificate *   newSelfSignedRSA_TlsCertificate(int rsaBits, iDate validUntil,
                                                     const iTlsCertificateName *namesNullTerminatedArray);
+iTlsCertificate *   copy_TlsCertificate         (const iTlsCertificate *);
 
 iBool               isEmpty_TlsCertificate      (const iTlsCertificate *);
 iBool               hasPrivateKey_TlsCertificate(const iTlsCertificate *);
@@ -99,8 +100,10 @@ iBool               verifyIp_TlsCertificate     (const iTlsCertificate *, const
 iBool               equal_TlsCertificate        (const iTlsCertificate *, const iTlsCertificate *);
 iString *           pem_TlsCertificate          (const iTlsCertificate *);
 iString *           privateKeyPem_TlsCertificate(const iTlsCertificate *);
-iBlock *            fingerprint_TlsCertificate  (const iTlsCertificate *);
-iBlock *            privateKeyFingerprint_TlsCertificate(const iTlsCertificate *d);
+
+iBlock *            fingerprint_TlsCertificate          (const iTlsCertificate *);
+iBlock *            publicKeyFingerprint_TlsCertificate (const iTlsCertificate *);
+iBlock *            privateKeyFingerprint_TlsCertificate(const iTlsCertificate *);
 
 /*----------------------------------------------------------------------------------------------*/
 
@@ -108,16 +111,18 @@ iDeclareClass(TlsRequest)
 iDeclareObjectConstruction(TlsRequest)
 
 iDeclareNotifyFunc    (TlsRequest, ReadyRead)
+iDeclareNotifyFuncArgs(TlsRequest, Sent, size_t sent, size_t toSend)
 iDeclareNotifyFunc    (TlsRequest, Finished)
 
 iDeclareAudienceGetter(TlsRequest, readyRead)
+iDeclareAudienceGetter(TlsRequest, sent)
 iDeclareAudienceGetter(TlsRequest, finished)
 
 enum iTlsRequestStatus {
     initialized_TlsRequestStatus,
     submitted_TlsRequestStatus,
     finished_TlsRequestStatus,
-    error_TlsRequestStatus
+    error_TlsRequestStatus,
 };
 
 void        setHost_TlsRequest          (iTlsRequest *, const iString *hostName, uint16_t port);
@@ -134,8 +139,12 @@ size_t                  receivedBytes_TlsRequest    (const iTlsRequest *);
 enum iTlsRequestStatus  status_TlsRequest           (const iTlsRequest *);
 const iString *         errorMessage_TlsRequest     (const iTlsRequest *);
 const iTlsCertificate * serverCertificate_TlsRequest(const iTlsRequest *);
+iBool                   isVerified_TlsRequest       (const iTlsRequest *);
+
+typedef iBool (*iTlsRequestVerifyFunc)(iTlsRequest *, const iTlsCertificate *, int depth);
 
 void        setCACertificates_TlsRequest(const iString *caFile, const iString *caPath);
 void        setCiphers_TlsRequest       (const char *cipherList);
+void        setVerifyFunc_TlsRequest    (iTlsRequestVerifyFunc verifyFunc);
 
 iEndPublic
